@@ -11,10 +11,10 @@ proc import_run {} {
   global incvs
   global insvn
   global cvsglb
-
+  
   gen_log:log T "ENTER"
   
-  cvsroot_check [pwd]
+  lassign [cvsroot_check [pwd]] incvs insvn inrcs ingit
   if {$incvs} {
     cvsok "This directory is already in CVS.\nCan\'t import here!" .import
     gen_log:log T "LEAVE"
@@ -24,7 +24,7 @@ proc import_run {} {
     gen_log:log T "LEAVE"
     return
   }
-
+  
   # This is just a default.  The user can change it.
   set cvsglb(newcode) [file tail $cwd]
   
@@ -35,31 +35,31 @@ proc import_run {} {
     gen_log:log T "LEAVE"
     return
   }
-
+  
   # Give it a default.  This is what you get without the -b option.
   set cvsglb(newvers) 1.1.1
   set cvsglb(newdir) $cvsglb(newcode)
-
+  
   toplevel .import
   grab set .import
-
+  
   frame .import.top
-
+  
   message .import.top.explain -justify left -width 500 -relief groove \
     -text "This will import the current directory and its sub-directories\
           into CVS, creating a new module.  If you haven't created a CVS repository,\
-          you must do that first with \"cvs init.\""
+      you must do that first with \"cvs init.\""
   label .import.top.lnewcode -text "New Module Name"  -anchor w
   label .import.top.lnewdir  -text "New Module path relative to \$CVSROOT" -anchor w
   label .import.top.lnewdesc -text "Descriptive Title" -anchor w
   label .import.top.lnewvers  -text "Version Number" -anchor w
-
-  entry .import.top.tnewcode -textvariable cvsglb(newcode) -width 40 
+  
+  entry .import.top.tnewcode -textvariable cvsglb(newcode) -width 40
   entry .import.top.tnewdir -textvariable cvsglb(newdir) -width 40
   entry .import.top.tnewdesc -textvariable cvsglb(newdesc) -width 40
   entry .import.top.tnewvers -textvariable cvsglb(newvers) -width 40
   
-
+  
   grid .import.top.explain -column 0 -row 0 -columnspan 3 -sticky ew
   grid .import.top.lnewcode -column 0 -row 1 -sticky w
   grid .import.top.tnewcode -column 1 -row 1 -sticky ew
@@ -69,31 +69,31 @@ proc import_run {} {
   grid .import.top.tnewdesc -column 1 -row 3 -sticky ew
   grid .import.top.lnewvers -column 0 -row 4 -sticky w
   grid .import.top.tnewvers -column 1 -row 4 -sticky ew
-
+  
   frame .import.down -relief groove -border 2
   button .import.down.ok -text "OK" \
-    -command {
-      grab release .import
-      wm withdraw .import
-      do_import
-    }
+      -command {
+    grab release .import
+    wm withdraw .import
+    do_import
+  }
   button .import.down.quit -text "Cancel" \
-    -command {
-      grab release .import
-      wm withdraw .import
-    }
-
+      -command {
+    grab release .import
+    wm withdraw .import
+  }
+  
   pack .import.down -side bottom -expand yes -fill x
   pack .import.top -side top -expand yes -fill x
   pack .import.down.ok -side left -expand yes
   pack .import.down.quit -side left -expand yes
-
+  
   # Needed for slower framebuffers
   #tkwait visibility .import
-
+  
   wm title .import "Create a New Module"
   wm minsize .import 1 1
-
+  
   gen_log:log T "LEAVE"
 }
 
@@ -106,10 +106,10 @@ proc do_import {} {
   global modval
   global modtitle
   global ExModList ExModDirList
-
+  
   gen_log:log T "ENTER"
   set imdir [pwd]
-
+  
   # Error checks
   if { $cvsglb(newcode) == "" } {
     cvsok "You must type in a new module name." .import
@@ -122,14 +122,14 @@ proc do_import {} {
   
   # We may have gotten here before opening the module browser
   if {! [info exists modlist_sorted]} {
-     modbrowse_run cvs
+    modbrowse_run
   }
   # Make sure it isn't a duplicate key
   foreach {key value} [array get modval] {
-     if { $cvsglb(newcode) == $key } {
-        cvsok "$cvsglb(newcode) is not a new Module" .import
-	return 1
-     }
+    if { $cvsglb(newcode) == $key } {
+      cvsok "$cvsglb(newcode) is not a new Module" .import
+      return 1
+    }
   }
   
   # See if all apropriate Directories in newdirname exist.  CVS import will
@@ -142,34 +142,34 @@ proc do_import {} {
       lappend knowndirs [lindex $idx 0]
     }
     gen_log:log D "looking for $pathname in known directories ($knowndirs)"
-    if {[lsearch -exact $knowndirs $pathname] == -1} {
+    if {$pathname ni $knowndirs} {
       set need_Dir 1
     }
   }
-
+  
   # Make a baseline tag
   set versions [split $cvsglb(newvers) ".,/ -"]
   set baseline "baseline-[join $versions {_}]"
-
+  
   set commandline "$cvs -d \"$cvscfg(cvsroot)\" import -m \"Imported using TkCVS\""
   # Let it default to 1.1.1 or you will have big problems later from cvs.
   #if {$cvsglb(newvers) != ""} {
-    #append commandline " -b 1.1.1"
+  #append commandline " -b 1.1.1"
   #}
   append commandline " \"$cvsglb(newdir)\" IMPORT $baseline"
-
+  
   set v [viewer::new "Import Module"]
   $v\::log "\nCVS Import\n"
   $v\::do "$commandline"
   $v\::wait
   update
-
+  
   # Update the modules file.
   set commandline "$cvs -d $cvscfg(cvsroot) -w checkout CVSROOT/modules"
   $v\::log "\nCheckout New Module\n"
   $v\::do "$commandline"
   $v\::wait
-
+  
   cd CVSROOT
   gen_log:log F "CD [pwd]"
   set modfile [open modules a]
@@ -192,9 +192,9 @@ proc do_import {} {
   set commandline "$cvs -d $cvscfg(cvsroot) -Q release -d CVSROOT"
   $v\::do "$commandline"
   $v\::wait
-
-  modbrowse_run cvs
-
+  
+  modbrowse_run
+  
   # Now check out the new module
   cd ..
   gen_log:log F "CD [pwd]"
@@ -202,11 +202,11 @@ proc do_import {} {
   # Otherwise checkout won't do the whole tree.
   gen_log:log F "MOVE $imdir $imdir.orig"
   if {[file isdirectory $imdir.orig]} {
-     file delete -force -- $imdir.orig
+    file delete -force -- $imdir.orig
   }
   file rename $imdir $imdir.orig
   set commandline \
-          "$cvs -d $cvscfg(cvsroot) checkout -R \"$cvsglb(newcode)\""
+      "$cvs -d $cvscfg(cvsroot) checkout -R \"$cvsglb(newcode)\""
   #gen_log:log C "$commandline"
   $v\::log "\nCVS Checkout\n"
   $v\::do "$commandline"
@@ -218,14 +218,14 @@ proc do_import {} {
   
   set ckmoddir $cwd
   if { $cvsglb(newcode) != [file tail $cwd] } {
-     set ckmoddir [file join [file dirname $cwd] $cvsglb(newcode)]
-  }  
+    set ckmoddir [file join [file dirname $cwd] $cvsglb(newcode)]
+  }
   if { [catch "cd $ckmoddir" err]} {
     cvsok "$err" .import
   } else {
     gen_log:log F "CD [pwd]"
   }
- 
+  
   if {$cvscfg(auto_status)} {
     setup_dir
   }
