@@ -1,7 +1,7 @@
 #!/bin/sh
 #-*-tcl-*-
 # the next line restarts using wish \
-exec wish "$0" -- ${1+"$@"}
+    exec wish "$0" -- ${1+"$@"}
 
 #
 # TkCVS Main program -- A Tk interface to CVS.
@@ -18,12 +18,12 @@ if {[info exists starkit::topdir]} {
 }
 
 if {! [info exists tk_version] } {
-   puts "Initialization failed"
-   exit 1
+  puts "Initialization failed"
+  exit 1
 }
 
-if {$tk_version < 8.4} {
-  puts "TkCVS requires Tcl/Tk 8.4 or better!"
+if {$tk_version < 8.5} {
+  puts "TkCVS requires Tcl/Tk 8.5 or better!"
   exit 1
 }
 
@@ -32,9 +32,9 @@ if {[info exists starkit::topdir]} {
   set ScriptBin $starkit::topdir
 } else {
   if {[info exists TclRoot]} {
-     # Perhaps we are being sourced recursively.
-     # That would be bad.
-     return
+    # Perhaps we are being sourced recursively.
+    # That would be bad.
+    return
   }
   set Script [info script]
   set ScriptTail [file tail $Script]
@@ -57,7 +57,7 @@ if {[info exists starkit::topdir]} {
   
   # allow runtime replacement
   if {[info exists env(TCLROOT)]} {
-   set TclRoot $env(TCLROOT)
+    set TclRoot $env(TCLROOT)
   }
   #puts "TclRoot $TclRoot"
 }
@@ -73,7 +73,7 @@ set cvscfg(bitmapdir) [file join $TclRoot tkcvs bitmaps]
 #puts "TCDIR $TCDIR"
 #puts "BITMAPDIR $cvscfg(bitmapdir)"
 
-set cvscfg(version) "8.2.3"
+set cvscfg(version) "9.3.2"
 
 if {! [info exists cvscfg(editorargs)]} {
   set cvscfg(editorargs) {}
@@ -91,11 +91,20 @@ set maxdirs 15
 set dirlist {}
 set totaldirs 0
 
+# Orient ourselves
 if { [info exists env(HOME)] } {
   set cvscfg(home) $env(HOME)
 } else {
   set cvscfg(home) "~"
 }
+if { [info exists env(USER)] } {
+  set cvscfg(user) $env(USER)
+} elseif { [info exists env(USERNAME)] } {
+  # Windows
+  set cvscfg(user) $env(USERNAME)
+} else {
+  set cvscfg(user) ""
+}
 
 # Read in defaults
 if {[file exists [file join $TCDIR tkcvs_def.tcl]]} {
@@ -109,13 +118,20 @@ set optfile [file join $cvscfg(home) .tkcvs]
 if {[file exists $optfile]} {
   catch {source $optfile}
 }
-::picklist::load
+set pickfile [file join $cvscfg(home) .tkcvs-picklists]
+if {[file exists $pickfile]} {
+  picklist_load
+}
+if {! [info exist cvsglb(directory)]} {
+  set cvsglb(directory) [pwd]
+}
+if {[info exists cvsglb(cvsroot)]} {
+  set cvscfg(cvsroot) [lindex $cvsglb(cvsroot) 0]
+}
 
 # Set some defaults
-set cvsglb(sort_pref) [list $cvscfg(sortcol) "-decreasing"]
 set cvsglb(commit_comment) ""
 set cvsglb(cvs_version) ""
-set cvsglb(svn_version) ""
 
 if {$cvscfg(use_cvseditor) && ![info exists cvscfg(terminal)]} {
   cvserror "cvscfg(terminal) is required if cvscfg(use_cvseditor) is set"
@@ -123,6 +139,7 @@ if {$cvscfg(use_cvseditor) && ![info exists cvscfg(terminal)]} {
 
 # Hilight colors.  Get the colorful ones.
 entry .testent
+wm withdraw .
 set cvsglb(textbg) white
 set cvsglb(textfg) black
 set cvsglb(hlbg) [lindex [.testent configure -selectbackground] 4]
@@ -132,6 +149,8 @@ if {$cvsglb(hlfg) eq {} } {
   set cvsglb(hlfg) [lindex [.testent configure -foreground] 4]
 }
 set cvscfg(listboxfont) [lindex [.testent configure -font] 4]
+#puts [font actual $cvscfg(listboxfont) -displayof .testent]
+#puts [font metrics $cvscfg(listboxfont) -displayof .testent]
 destroy .testent
 
 
@@ -143,35 +162,32 @@ if {$WSYS eq "x11"} {
   # If X11, see if we can sense our environment somehow
   label .testlbl -text "LABEL"
   if [get_cde_params] {
-    set theme_system "CDE" 
+    set theme_system "CDE"
     # Find out what the default gui font is
     if { ! [info exists cvscfg(guifont)] } {
       # Find out what the tk default is
       set cvscfg(guifont) [lindex [.testlbl configure -font] 4]
     }
     # Put the Help menu back on the right
-    if {$tk_version >= 8.5} {
-      tk::classic::restore menu
-    }
-    #set cvsglb(canvbg) [lindex [.testlbl configure -background] 4]
+    tk::classic::restore menu
+    # canvbg is for the treeviews, not the branch browser canvas
     set cvsglb(canvbg) $cvsglb(shadow)
   } elseif [get_gtk_params] {
     set theme_system "GTK"
     if { ! [info exists cvscfg(guifont)] } {
       set cvscfg(guifont) [lindex [.testlbl configure -font] 4]
-      if {$tk_version >= 8.5} {
-       font configure TkDefaultFont -size 9
-       set cvscfg(guifont) TkDefaultFont
-       option add *Menu.font $cvscfg(guifont)
-       option add *Label.font $cvscfg(guifont)
-       option add *Button.font $cvscfg(guifont)
-      }
+      font configure TkDefaultFont -size 9
+      set cvscfg(guifont) TkDefaultFont
+      option add *Menu.font $cvscfg(guifont)
+      option add *Label.font $cvscfg(guifont)
+      option add *Button.font $cvscfg(guifont)
     }
     # in KDE or Gnome or some such.  It rather rudely sets all the Tk
     # backgrounds the same which I don't like, so I'm going to use the same
     # trick I use for CDE to give the canvases a little shading.
     set cvsglb(bg) [lindex [.testlbl cget -background] 0]
     set cvsglb(fg) [lindex [.testlbl cget -foreground] 0]
+    # canvbg is for the treeviews, not the branch browser canvas
     set cvsglb(canvbg) [rgb_shadow $cvsglb(bg)]
   } else {
     set bg  [lindex [.testlbl cget -background] 0]
@@ -180,23 +196,23 @@ if {$WSYS eq "x11"} {
     set hlfg "#ffffff"
     set textbg "#ffffff"
     set textfg "#000000"
-
+    
     shades $bg
-
+    
     set cvsglb(bg) $bg
     set cvsglb(fg) $fg
     set cvsglb(textbg) $textbg
     set cvsglb(textfg) $textfg
     set cvsglb(hlbg) $hlbg
     set cvsglb(hlfg) $hlfg
-
+    
     option add *Canvas.Background $cvsglb(shadow)
     option add *Canvas.Foreground black
     option add *Entry.Background $textbg
     option add *Entry.Foreground $textfg
     option add *Entry.selectBackground $hlbg
     option add *Entry.selectForeground $hlfg
-    option add *Entry.readonlyBackground $bg
+    option add *Entry.readonlyBackground $cvsglb(lighter)
     option add *Listbox.background $textbg
     option add *Listbox.selectBackground $hlbg
     option add *Listbox.selectForeground $hlfg
@@ -206,34 +222,29 @@ if {$WSYS eq "x11"} {
     option add *Text.selectForeground $hlfg
     option add *Button.activeForeground $fg
     option add *Menu.activeForeground $fg
-
+    option add *Checkbutton.Background $bg
+    
     # checkbuttons and radiobuttons
-    if {$tk_version >= 8.5} {
-       option add *Menu.selectColor $fg
-       option add *Checkbutton.selectColor "#ffffff"
-       option add *Radiobutton.selectColor "#ffffff"
-     } else {
-       option add *selectColor $hlbg
-     }
-
+    option add *Menu.selectColor $fg
+    option add *Checkbutton.selectColor "#ffffff"
+    option add *Radiobutton.selectColor "#ffffff"
+    
     if { ! [info exists cvscfg(guifont)] } {
       set cvscfg(guifont) [lindex [.testlbl configure -font] 4]
-      if {$tk_version >= 8.5} {
-       # This makes it look like tk8.4.  Want?  dunno.
-       font configure TkHeadingFont -size 9
-       set cvscfg(guifont) TkHeadingFont
-       option add *Menu.font $cvscfg(guifont)
-       option add *Label.font $cvscfg(guifont)
-       option add *Button.font $cvscfg(guifont)
-      }
+      # This makes it look more classic
+      font configure TkHeadingFont -size 9
+      set cvscfg(guifont) TkHeadingFont
+      option add *Menu.font $cvscfg(guifont)
+      option add *Label.font $cvscfg(guifont)
+      option add *Button.font $cvscfg(guifont)
     }
   }
   destroy .testlbl
-
+  
   if {! [info exists cvscfg(dialogfont)]} {
     set cvscfg(dialogfont) $cvscfg(guifont)
   }
-
+  
   if {$theme_system == "CDE"} {
     # This makes it consistent with the rest of the CDE interface
     option add *Menu.font $cvscfg(guifont)
@@ -247,67 +258,34 @@ if {$WSYS eq "x11"} {
   # Find out what the tk default is
   set cvscfg(guifont) [lindex [.testlbl configure -font] 4]
   set cvscfg(dialogfont) $cvscfg(guifont)
-
-  set cvsglb(canvbg) [lindex [.testlbl configure -background] 4]
+  
   set cvsglb(bg) [lindex [.testlbl cget -background] 0]
   set cvsglb(fg) [lindex [.testlbl cget -foreground] 0]
+  # canvbg is for the treeviews, not the branch browser canvas
   set cvsglb(canvbg) [rgb_shadow $cvsglb(bg)]
   destroy .testlbl
   if {$WSYS eq "aqua"} {
-    # keep evertyhing from being blinding white
+    # Keep everything from being blinding white
+    set arbitrarybg "#dddddd"
+    set lighterbg "#eeeeee"
+    option add *Frame.background $arbitrarybg userDefault
+    option add *Label.background $arbitrarybg userDefault
+    option add *Entry.highlightBackground $arbitrarybg userDefault
+    option add *Entry.readonlyBackground $lighterbg userDefault
+    option add *Canvas.highlightBackground #fefefe userDefault
+    option add *Message.Background $arbitrarybg userDefault
+    option add *Checkbutton.Background $arbitrarybg userDefault
+    option add *Radiobutton.Background $arbitrarybg userDefault
     # button highlightbackground has to be the same as background
     # or else there are little white boxes around the button "pill"
-    option add *background #ebebeb userDefault
-    option add *Button.highlightBackground #ebebeb userDefault
-    # That totally screws up the menus unless you fix it with this
-    option add *Menu.Background white
-    option add *Menu.Foreground black
-
-    option add *Entry.HighlightThickness 2 userDefault
-    option add *Entry.highlightBackground $cvsglb(hlbg) userDefault
-    option add *Canvas.background #eeeeee userDefault
-    option add *Entry.background #ffffff userDefault
-    option add *Text.background white userDefault
+    option add *Button.highlightBackground $arbitrarybg userDefault
+    # canvbg is for the treeviews, not the branch browser canvas
+    set cvsglb(canvbg) $lighterbg
   }
 }
 
-
-# Fonts for the canvas "listboxes"
-set cvscfg(flashfont) $cvscfg(listboxfont)
-set fsiz [lindex $cvscfg(listboxfont) 1]
-set lbf [font actual $cvscfg(listboxfont)]
-#puts "$tcl_platform(os) $tk_patchLevel   [winfo server .]"
-#puts "listboxfont: $cvscfg(listboxfont)"
-#puts "actual listboxfont: $lbf"
-set ffam [lindex $lbf 1]
-set fsiz [lindex $lbf 3]
-regsub -- {-} $fsiz {} fsiz
-
-if {[tk windowingsystem] eq "x11"} {
-  if {$tk_version >= 8.5} {
-    # Put the help menu back on the right
-    #tk::classic::restore menu
-
-    if {$tcl_platform(os) eq "Linux" && [lindex $tk_patchLevel 2] < 8} {
-      # FIXME: check for X.Org in server string?
-      #puts [winfo server .]
-      # Overstrike and underline fonts in Xorg were too big from 8.5.0 to 8.5.7
-      set fsiz -$fsiz
-    }
-  }
-}
-
-set cvscfg(flashfont) [list $ffam $fsiz underline]
-#puts "requested flashfont: $cvscfg(flashfont)"
-#puts "actual flashfont: [font actual $cvscfg(flashfont)]"
-#puts "try underline: [font actual $cvscfg(flashfont) -underline]"
-# Prefer underline, but it isn't working at all in tk8.5.0 on linux
-if {! [font actual $cvscfg(flashfont) -underline]} {
-  puts "Underline font not working.  Trying $ffam $fsiz bold"
-  puts " (known problem in Tk 8.5.0 on Linux)"
-  set cvscfg(flashfont) [list $ffam $fsiz bold]
-}
-#puts "final flashfont: $cvscfg(flashfont)"
+# Suppress tearoffs in menubars
+option add *tearOff 0
 
 option add *ToolTip.background  "LightGoldenrod1" userDefault
 option add *ToolTip.foreground  "black" userDefault
@@ -318,6 +296,16 @@ option add *Dialog.msg.font $cvscfg(dialogfont) userDefault
 # Sometimes we'll want to override this but let's set a default
 option add *Message.font $cvscfg(dialogfont) userDefault
 
+if {$WSYS eq "x11"} {
+  ttk::style configure TCombobox -arrowsize 16
+  # Header padding has no effect on aqua, but it works on X11
+  ttk::style configure Treeview.Heading -padding {4 0}
+}
+# And, background seems to have no effect on X11
+ttk::style configure Treeview -font $cvscfg(listboxfont) -background $cvsglb(canvbg) \
+    -fieldbackground $cvsglb(canvbg)
+ttk::style configure Treeview.Heading -font $cvscfg(listboxfont) -background $cvsglb(bg)
+ttk::style configure Treeview.Cell -padding {2 0}
 
 # Initialize logging (classes are C,F,T,D)
 if { ! [info exists cvscfg(log_classes)] } {
@@ -332,12 +320,8 @@ if { ! [info exists cvscfg(logging)] } {
 if {$cvscfg(logging)} {
   gen_log:init
 }
-
-#
-# Add directory where we last ran to the menu list
-if { ! [info exists cvscfg(lastdir)] } {
-  set cvscfg(lastdir) [pwd]
-}
+# Detect whether we're in a revision-controlled directory
+lassign [cvsroot_check [pwd]] incvs insvn inrcs ingit
 
 #
 # Command line options
@@ -346,6 +330,7 @@ set usage "Usage:"
 append usage "\n tkcvs \[-dir <directory>\] \[-root <cvsroot>\] \[-win workdir|module|merge\]"
 append usage "\n tkcvs \[-dir <directory>\] \[-root <cvsroot>\] \[-log|blame <file>\]"
 append usage "\n tkcvs <file> - same as tkcvs -log <file>"
+append usage "\n tkcvs <dir>  - same as tkcvs -dir <file>"
 for {set i 0} {$i < [llength $argv]} {incr i} {
   set arg [lindex $argv $i]
   set val [lindex $argv [expr {$i+1}]]
@@ -381,10 +366,22 @@ for {set i 0} {$i < [llength $argv]} {incr i} {
       puts $usage
       exit 0
     }
-    {\w*} {
+    {^\.*\w*} {
       # If a filename is provided as an argument, assume -log
-      set cvscfg(startwindow) log
-      set lcfile $arg; incr i
+      # except if it's a directory and it's CVS, which doesn't
+      # version directories
+      if {($insvn || $ingit)} {
+        set cvscfg(startwindow) log
+        set lcfile $arg; incr i
+      } else {
+        if {[file isdirectory $arg]} {
+          set dir $arg
+          cd $arg
+        } else {
+          set cvscfg(startwindow) log
+          set lcfile $arg; incr i
+        }
+      }
     }
     default {
       puts $usage
@@ -400,125 +397,89 @@ if {[info exists lcfile]} {
   cd $d
 }
 
-# If CVSROOT envvar is set, use it
-if { ! [info exists cvscfg(cvsroot)] } {
-  if { ! [info exists env(CVSROOT)] } {
-    #puts "warning: your \$CVSROOT environment variable is not set."
-    set cvscfg(cvsroot) ""
-  } else {
-    set cvscfg(cvsroot) $env(CVSROOT)
-  }
-}
-# This helps with Samba-mounted CVS repositories
-# And also completely messes up SVN repositories
-#set cvscfg(cvsroot) [file join $cvscfg(cvsroot)]
-# If SVNROOT is set, use that instead.  SVNROOT isn't
-# known by Subversion itself, so if it's set we must have
-# done it for the present purpose
-if {! [info exists cvscfg(svnroot)] } {
-  if { [info exists env(SVNROOT)] } {
-    set cvscfg(svnroot) $env(SVNROOT)
-  } else {
-    set cvscfg(svnroot) ""
-  }
-}
-set cvsglb(root) $cvscfg(cvsroot)
-if {$cvscfg(svnroot) != ""} {
-  set cvsglb(root) $cvscfg(svnroot)
-}
-# Thought better of saving this
-catch unset cvscfg(svnconform_seen)
-
 if {![info exists cvscfg(ignore_file_filter)]} {
   set cvscfg(ignore_file_filter) ""
 }
-# Remember what the setting was.  We'll have to restore it after
-# leaving a directory with a .cvsignore file.
-set cvsglb(default_ignore_filter) $cvscfg(ignore_file_filter)
+if {[info exists cvscfg(file_filter)]} {
+  unset cvscfg(file_filter)
+}
+if {![info exists cvscfg(show_file_filter)]} {
+  set cvscfg(show_file_filter) "*"
+}
 
-#foreach c [lsort [array names cvscfg]] {
-  #gen_log:log D "cvscfg($c) $cvscfg($c)"
-#}
+load_all_images
 
-# Load the images that are used in more than one module
-image create photo Log \
-  -format gif -file [file join $cvscfg(bitmapdir) log.gif]
-image create photo Checkout \
-  -format gif -file [file join $cvscfg(bitmapdir) checkout.gif]
-image create photo CheckoutOpts \
-  -format gif -file [file join $cvscfg(bitmapdir) checkout_opts.gif]
-image create photo Export \
-  -format gif -file [file join $cvscfg(bitmapdir) export.gif]
-image create photo Tag \
-  -format gif -file [file join $cvscfg(bitmapdir) tag.gif]
-image create photo Branchtag \
-   -format gif -file [file join $cvscfg(bitmapdir) branchtag.gif]
-image create photo Import \
-   -format gif -file [file join $cvscfg(bitmapdir) import.gif]
-image create photo Mergebranch \
-  -format gif -file [file join $cvscfg(bitmapdir) newmerge_simple.gif]
-image create photo Mergediff \
-  -format gif -file [file join $cvscfg(bitmapdir) newmerge.gif]
-image create photo Man \
-  -format gif -file [file join $cvscfg(bitmapdir) man.gif]
-
-
-
-set incvs 0
-set insvn 0
-set inrcs 0
+set cvsglb(root) ""
+set cvsglb(vcs) ""
 
 # Create a window
 # Start with Module Browser
 if {[string match {mod*} $cvscfg(startwindow)]} {
   wm withdraw .
-  foreach {incvs insvn inrcs} [cvsroot_check [pwd]] { break }
-
+  # If we're in a version-controlled directory, open that repository
   if {$insvn} {
     set cvsglb(root) $cvscfg(svnroot)
-    modbrowse_run svn
+    set cvsglb(vcs) svn
+  } elseif {$incvs} {
+    set cvsglb(root) $cvscfg(cvsroot)
+    set cvsglb(vcs) cvs
+  } elseif {$ingit} {
+    set cvsglb(root) $cvscfg(url)
+    set cvsglb(vcs) git
   } else {
-    # We still don't know if it's SVN or CVS.  Let modbrowse_run figure out.
-    modbrowse_run
+    # We'll respect CVSROOT environment variable if it's set
+    if {[info exists env(CVSROOT)]} {
+      set cvsglb(root) $env(CVSROOT)
+      set cvscfg(cvsroot) $env(CVSROOT)
+      set cvsglb(vcs) cvs
+    }
   }
-# Start with Branch Browser
+  # Othewise we set it to the most recent saved in picklist
+  # which we've saved in cvscfg(cvsroot)
+  if {$cvsglb(root) == ""} {
+    set cvsglb(root) $cvscfg(cvsroot)
+  }
+  modbrowse_run
+  # Start with Branch Browser
 } elseif {$cvscfg(startwindow) == "log"} {
   if {! [file exists $lcfile]} {
     puts "ERROR: $lcfile doesn't exist!"
     exit 1
   }
   wm withdraw .
-  foreach {incvs insvn inrcs} [cvsroot_check [pwd]] { break }
   if {$incvs} {
-    cvs_branches \"$lcfile"\
+    cvs_branches [list $lcfile]
   } elseif {$inrcs} {
     set cwd [pwd]
     set module_dir ""
-    rcs_branches \"$lcfile\"
+    rcs_branches [list $lcfile]
   } elseif {$insvn} {
-    svn_branches \"$lcfile\"
+    svn_branches [list $lcfile]
+  } elseif {$ingit} {
+    git_branches [list $lcfile]
   } else {
-    puts "File doesn't seem to be in CVS, SVN, or RCS"
+    puts "File doesn't seem to be in CVS, SVN, RCS, or GIT"
   }
-# Start with Annotation Browser
+  # Start with Annotation Browser
 } elseif {$cvscfg(startwindow) == "blame"} {
   if {! [file exists $lcfile]} {
     puts "ERROR: $lcfile doesn't exist!"
     exit 1
   }
   wm withdraw .
-  foreach {incvs insvn inrcs} [cvsroot_check [pwd]] { break }
   if {$incvs} {
-    cvs_annotate "" \"$lcfile"\
+    cvs_annotate $current_tagname [list $lcfile]
   } elseif {$insvn} {
-    svn_annotate "" \"$lcfile\"
+    svn_annotate rBASE [list $lcfile]
+  } elseif {$ingit} {
+    read_git_dir .
+    git_annotate $current_tagname [list $lcfile]
   } else {
-    puts "File doesn't seem to be in CVS or SVN"
+    puts "File doesn't seem to be in CVS, SVN, or GIT"
   }
-# Start with Directory Merge
+  # Start with Directory Merge
 } elseif {[string match {mer*} $cvscfg(startwindow)]} {
   wm withdraw .
-  foreach {incvs insvn inrcs} [cvsroot_check [pwd]] { break }
   if {$incvs} {
     cvs_joincanvas
   } elseif {$insvn} {
@@ -526,8 +487,8 @@ if {[string match {mod*} $cvscfg(startwindow)]} {
   } else {
     puts "Directory doesn't seem to be in CVS or SVN"
   }
-# The usual way, with the Workdir Browser
+  # The usual way, with the Workdir Browser
 } else {
-  workdir_setup
+  setup_dir
 }
 
